/***************************************************************************
 *   Copyright (C) 2007 by Pierre Marchand   *
 *   pierre@oep-h.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef TYPOTEK_H
#define TYPOTEK_H

#include <QMainWindow>
#include <QCloseEvent>
#include <QMap>
#include <QFile>
#include <QDir>

class QAction;
class QMenu;
class QTextEdit;
class MainViewWidget;
class FontItem;
class TypotekAdaptator;



class typotek:public QMainWindow
{
		Q_OBJECT

	public:
		typotek();
		void initMatrix();
		~typotek();

	protected:
		void closeEvent ( QCloseEvent *event );
		void keyPressEvent ( QKeyEvent * event ) ;

	private slots:
		void newFile();
		void open();
		void print();
		void fontBook();
		void popupTagsetEditor();
		void slotActivateCurrents();
		void slotDeactivateCurrents();

// 		bool saveAs();
		void about();
		void help();
// 		void documentWasModified();
	public slots:
		bool save();
	signals:
		void tagAdded(QString);
		void relayStartingStep(QString, int, QColor);

	private:
		void createActions();
		void createMenus();
		void createToolBars();
		void createStatusBar();
		void readSettings();
		void writeSettings();
		bool maybeSave();
		void loadFile ( const QString &fileName );
		bool saveFile ( const QString &fileName );
		void setCurrentFile ( const QString &fileName );
		QString strippedName ( const QString &fullFileName );
		void initDir();
		void doConnect();

		void checkOwnDir();
		void fillTagsList();

		QTextEdit *textEdit;
		QString curFile;

		QMenu *fileMenu;
		QMenu *editMenu;
		QMenu *helpMenu;
		QToolBar *fileToolBar;
		QToolBar *editToolBar;
		QAction *newAct;
		QAction *openAct;
		QAction *saveAct;
		QAction *saveAsAct;
		QAction *exitAct;
		QAction *cutAct;
		QAction *copyAct;
		QAction *pasteAct;
		QAction *aboutAct;
		QAction *aboutQtAct;
		QAction *printAct;
		QAction *fontBookAct;
		QAction *tagsetAct;
		QAction *activCurAct;
		QAction *deactivCurAct;
		QAction *helpAct;

		MainViewWidget *theMainView;

		QFile fontsdata;
		QDir ownDir;
		QDir managedDir;

		QMap<QString, QStringList> tagsMap;
		QMap<QString, QStringList> tagSetMap;
		
		QList<FontItem*> fontMap;
		QMap<QString, FontItem*> realFontMap;
		
		TypotekAdaptator *actAdaptator;
		
		QString m_sampleText;
	public:
		FontItem* getFont ( int i ) ;
		FontItem* getFont ( QString s );
		QList<FontItem*> getAllFonts() {return fontMap;};
		QList<FontItem*> getFonts ( QString pattern, QString field );
		QList<FontItem*> getCurrentFonts();
		
		void addTagMapEntry(QString key, QStringList value){tagsMap[key] = value;};
		void addTagSetMapEntry(QString key, QStringList value){tagSetMap[key] = value;};
		void removeTagFromSet(QString set, QString tag){tagSetMap[set].removeAll(tag);};
		void addTagToSet(QString set, QString tag){tagSetMap[set].append(tag);};
		void removeTagset(QString key){tagSetMap.remove(key);};
		QStringList tagsets(){return tagSetMap.keys();};
		QStringList tagsOfSet(QString set){return tagSetMap[set];};
		
		static QStringList tagsList;
		TypotekAdaptator *adaptator(){return actAdaptator;};
		
		static typotek* instance;
		static typotek* getInstance(){return instance;};
		
		QString getManagedDir(){return managedDir.absolutePath();};
		
		void setSampleText(QString s){m_sampleText = s;};
		QString sampleText(){return m_sampleText;};
};



#endif
